
SLAVE_CODE      EQU     1       ; DO NOT CHANGE THESE VALUES
MASTER_CODE     EQU     2       ; OR COMM_CHECK_OTHER_STATUS BREAKS!!

SLAVE_INIT      EQU     $80
MASTER_INIT     EQU     $81


        SECTION "Comm Code",HOME

;Initialize comm stuff

comm_not_ready:
        xor     a
        ld      [rd],a
        ld      [SIODone],a
        ld      [SIOType],a    ; Indicate serial link down
        ld      [VBlankDone],a

        ld      a,SLAVE_CODE
        ldh     [rSB],a

        ld      a,SLAVE_INIT
        ldh     [rSC],a

        ret

; Call this to start a 2 player game as a master.

comm_start_master_game:
        ld      a,MASTER_CODE
        ldh     [rSB],a

        ld      a,MASTER_INIT
        ldh     [rSC],a

        ret

; Check other unit status.
; Exit: A = 0 if other unit not ready.

comm_check_other_status:
        ld      a,[rd]
        cp      MASTER_CODE     ; Master code received?
        ret     z               ; yes

        cp      SLAVE_CODE      ; Slave code received?
        ret     z               ; yes

        xor     a               ; A = 0
        ret

; Communications transfer
; Entry: 'td' is byte to transmit
; Exit:  'rd' is byte to receive
;      All data transmitted & received is
;     delayed by one VBlank!!!!

comm_transfer:
        ld      a,[SIOType]
        cp      MASTER_CODE     ; Are we a master?
        ret     nz              ; no

        ld      a,MASTER_INIT
        ldh     [rSC],a
        ret

; Synchronize both game units

Synchronize:
        ld      a,[SIOType]
        cp      MASTER_CODE     ; Are we a master?
        jr      z,.master       ; yes

.slave: ld      a,[SIODone]
        or      a               ; Have we received serial byte?
        jr      z,.slave        ; no, wait

        xor     a
        ld      [SIODone],a
        ret

.master:
        ld      a,[VBlankDone]
        or      a               ; Has VBlank occured yet?
        jr      z,.master       ; no, wait

        xor     a
        ld      [VBlankDone],a
        ret

; *** Serial Interrupt Routine ***

SIOIntr:
        push    af

        ldh     a,[rSB]
        ld      [rd],a                  ; rd <- [SB]

        ld      a,[SIOType]
        or      a                       ; Serial link established?
        jr      nz,.linkup              ; yes

        ldh     a,[rSB]
        cp      SLAVE_CODE              ; was a slave code received?
        jr      z,.settype              ; yes
        cp      MASTER_CODE             ; was a master code received?
        jr      z,.settype              ; yes

        xor     a
        ld      [rd],a

        ld      a,SLAVE_CODE
        ldh     [rSB],a

        ld      a,SLAVE_INIT
        ldh     [rSC],a

        jr      .exit


.linkup:
        ld      a,[td]
        ldh     [rSB],a                 ; [SB] <- td

        ld      a,[SIOType]
        cp      MASTER_CODE             ; are we a master?
        jr      z,.exit                 ; yes, we're done

        ld      a,SLAVE_INIT
        ldh     [rSC],a

        ld      a,1
        ld      [SIODone],a

.exit:
        pop     af
        reti

.settype:
        xor     3               ; 2 -> 1, 1 -> 2
        ld      [SIOType],a     ; set serial type

        ld      a,[td]
        ldh     [rSB],a

        ld      a,SLAVE_INIT
        ldh     [rSC],a

        pop     af
        reti


        SECTION "UtilityVars",BSS

rd              DS      1
td              DS      1
SIODone         DS      1
SIOInit         DS      1
SIOType         DS      1       ; holds MASTER_CODE or SLAVE_CODE

